﻿using ComputersShopContracts.BindingModels;
using ComputersShopContracts.SearchModels;
using ComputersShopContracts.StoragesContracts;
using ComputersShopContracts.ViewModels;
using ComputersShopDataBaseImplement.Models;
using ComputersShopDataModels.Models;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ComputersShopDataBaseImplement.Implements
{
	public class ShopStorage : IShopStorage
	{
		public ShopViewModel? Delete(ShopBindingModel model)
		{
			using var context = new ComputersShopDataBase();
			var element = context.Shops
				.Include(x => x.shopComputers)
				.FirstOrDefault(rec => rec.Id == model.Id);
			if (element != null)
			{
				context.Shops.Remove(element);
				context.SaveChanges();
				return element.GetViewModel;
			}
			return null;
		}

		public ShopViewModel? GetElement(ShopSearchModel model)
		{
			if (string.IsNullOrEmpty(model.ShopName) && !model.Id.HasValue)
			{
				return null;
			}
			using var context = new ComputersShopDataBase();
			return context.Shops
				.Include(x => x.shopComputers)
				.ThenInclude(x => x.Computer)
				.FirstOrDefault(x => (!string.IsNullOrEmpty(model.ShopName) && x.ShopName == model.ShopName) || (model.Id.HasValue && x.Id == model.Id))?.GetViewModel;
		}

		public List<ShopViewModel> GetFilteredList(ShopSearchModel model)
		{
			if (string.IsNullOrEmpty(model.ShopName))
			{
				return new();
			}
			using var context = new ComputersShopDataBase();
			return context.Shops
				.Include(x => x.shopComputers)
				.ThenInclude(x => x.Computer)
				.Where(x => x.ShopName.Contains(model.ShopName))
				.ToList()
				.Select(x => x.GetViewModel)
				.ToList();
		}

		public List<ShopViewModel> GetFullList()
		{
			using var context = new ComputersShopDataBase();
			return context.Shops
				.Include(x => x.shopComputers)
				.ThenInclude(x => x.Computer)
				.ToList()
				.Select(x => x.GetViewModel)
				.ToList();
		}

		public ShopViewModel? Insert(ShopBindingModel model)
		{
			using var context = new ComputersShopDataBase();
			var newComputer = Shop.Create(context, model);
			if (newComputer == null)
			{
				return null;
			}
			context.Shops.Add(newComputer);
			context.SaveChanges();
			return newComputer.GetViewModel;
		}

		public bool SellComputers(IComputerModel model, int quantity)
		{
			using var context = new ComputersShopDataBase();
			using var transaction = context.Database.BeginTransaction();
			try
			{
				List<Shop> shopsWithComputer = context.Shops.Include(x => x.shopComputers).ThenInclude(x => x.Computer).Where(x => x.shopComputers.Any(x => x.ComputerId == model.Id)).ToList();
				foreach (var shop in shopsWithComputer)
				{
					int computerInShopCount = shop.Computers[model.Id].Item2;
					if (quantity - computerInShopCount >= 0)
					{
						quantity -= computerInShopCount;
						context.ShopComputers.Remove(shop.shopComputers.FirstOrDefault(x => x.ComputerId == model.Id)!);
						shop.Computers.Remove(model.Id);
					}
					else
					{
						shop.Computers[model.Id] = (model, computerInShopCount - quantity);
						quantity = 0;
						shop.UpdateComputers(context, new()
						{
							Id = shop.Id,
							Computers = shop.Computers
						});
					}
					if (quantity == 0)
					{
						context.SaveChanges();
						transaction.Commit();
						return true;
					}
				}
				transaction.Rollback();
				return false;
			}
			catch
			{
				transaction.Rollback();
				throw;
			}
		}

		public ShopViewModel? Update(ShopBindingModel model)
		{
			using var context = new ComputersShopDataBase();
			using var transaction = context.Database.BeginTransaction();
			try
			{
				var shop = context.Shops.FirstOrDefault(rec =>
				rec.Id == model.Id);
				if (shop == null)
				{
					return null;
				}
				shop.Update(model);
				context.SaveChanges();
				shop.UpdateComputers(context, model);
				transaction.Commit();
				return shop.GetViewModel;
			}
			catch
			{
				transaction.Rollback();
				throw;
			}
		}
	}
}
